/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AQDB_QUERY_BE_H
#define AQDB_QUERY_BE_H

#include <aqdatabase/aqdb_query.h>
#include <aqdatabase/aqdb_db_be.h>
#include <aqdatabase/aqdb_db.h>
#include <aqdatabase/aqdb_value.h>

#include <gwenhywfar/gwendate.h>
#include <gwenhywfar/inherit.h>


GWEN_INHERIT_FUNCTION_LIB_DEFS(AQDB_QUERY, AQDB_API)





typedef enum {
  AQDB_Query_Expr_OperatorUnknown=-1,
  AQDB_Query_Expr_OperatorNone=0,
  AQDB_Query_Expr_OperatorData=1,
  AQDB_Query_Expr_OperatorField,
  AQDB_Query_Expr_OperatorEqual,
  AQDB_Query_Expr_OperatorNotEqual,
  AQDB_Query_Expr_OperatorGreaterThan,
  AQDB_Query_Expr_OperatorGreaterThanOrEqual,
  AQDB_Query_Expr_OperatorLesserThan,
  AQDB_Query_Expr_OperatorLesserThanOrEqual,
  AQDB_Query_Expr_OperatorContains,
  AQDB_Query_Expr_OperatorMatches,
  AQDB_Query_Expr_OperatorIsSet,
  AQDB_Query_Expr_OperatorAreSet,
  AQDB_Query_Expr_OperatorIsClear,
  AQDB_Query_Expr_OperatorOr,
  AQDB_Query_Expr_OperatorAnd,
  AQDB_Query_Expr_OperatorXor,
} AQDB_QUERY_EXPR_OPERATOR;



struct AQDB_QUERY_EXPR {
  AQDB_QUERY *query;
  AQDB_QUERY_EXPR *left;
  AQDB_QUERY_EXPR_OPERATOR op;
  AQDB_QUERY_EXPR *right;

  AQDB_DATA_TYPE resultType;
  int fieldIndex;

  char *dataPtr;
  uint32_t dataLen;

  int64_t intData;

  union {
    AQDB_VALUE *value;
    GWEN_DATE *date;
    GWEN_TIME *timestamp;
  } typedPtr;


};



AQDB_API AQDB_QUERY_EXPR *AQDB_Query_Expr_new();
AQDB_API AQDB_QUERY_EXPR *AQDB_Query_Expr_Data_new(AQDB_QUERY *q, const char *ptr, uint32_t len);
AQDB_API AQDB_QUERY_EXPR *AQDB_Query_Expr_Field_new(AQDB_QUERY *q, int idx, AQDB_DATA_TYPE t);

AQDB_API void AQDB_Query_Expr_free(AQDB_QUERY_EXPR *expr);


typedef AQDB_DATA_TYPE GWENHYWFAR_CB(*AQDB_QUERY_GETFIELDTYPE_FN)(AQDB_QUERY *q,
                                                                  AQDB_QUERY_EXPR *expr,
                                                                  const char *name);
typedef int GWENHYWFAR_CB(*AQDB_QUERY_GETFIELDDATA_FN)(AQDB_QUERY *q, AQDB_QUERY_EXPR *expr);
typedef AQDB_QUERY_EXPR *GWENHYWFAR_CB(*AQDB_QUERY_MKFIELDEXPR_FN)(AQDB_QUERY *q, const char *name);



struct AQDB_QUERY {
  GWEN_INHERIT_ELEMENT(AQDB_QUERY)

  AQDB_QUERY_EXPR *expression;

  /* current object to compare against expression */
  void *object;

  /* dont access these directly */
  AQDB_QUERY_MKFIELDEXPR_FN mkFieldExprFn;
  AQDB_QUERY_GETFIELDDATA_FN getFieldDataFn;
};




AQDB_API AQDB_QUERY *AQDB_Query_new();

AQDB_API int AQDB_Query_SetString(AQDB_QUERY *q, const char *s);

AQDB_API int AQDB_Query_Calculate(AQDB_QUERY *q);

AQDB_API void AQDB_Query_SetMkFieldExprFn(AQDB_QUERY *q, AQDB_QUERY_MKFIELDEXPR_FN f);
AQDB_API void AQDB_Query_SetGetFieldDataFn(AQDB_QUERY *q, AQDB_QUERY_GETFIELDDATA_FN f);


#endif



