/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_VALUE_H
#define AQDB_VALUE_H

#include <aqdatabase/aqdb.h>

#include <gwenhywfar/buffer.h>

#include <stdio.h>


#ifdef __cplusplus
extern "C" {
#endif

typedef struct AQDB_VALUE AQDB_VALUE;


AQDB_API
AQDB_VALUE *AQDB_Value_new();

AQDB_API
AQDB_VALUE *AQDB_Value_dup(const AQDB_VALUE *ov);

AQDB_API
void AQDB_Value_free(AQDB_VALUE *v);

/**
 * This function reads a AQDB_VALUE from a string. Strings suitable as
 * arguments are those created by @ref AQDB_Value_toString or simple
 * floating point string (as in "123.45" or "-123.45").
 */
AQDB_API
AQDB_VALUE *AQDB_Value_fromString(const char *s);

/**
 * This function exports the value in a format which can be recognized
 * by the function @ref AQDB_Value_fromString. You should not make any
 * assumption about the format of the string created here.
 */
AQDB_API
void AQDB_Value_toString(const AQDB_VALUE *v, GWEN_BUFFER *buf);


AQDB_API
void AQDB_Value_toHumanReadableString(const AQDB_VALUE *v,
                                      GWEN_BUFFER *buf,
                                      int prec);

/**
 * This function does the same as @ref AQDB_Value_toHumanReadableString
 * but it prints the value using the current locale.
 */
AQDB_API
void AQDB_Value_toLocalHumanReadableString(const AQDB_VALUE *v,
                                           GWEN_BUFFER *buf,
                                           int prec);

AQDB_API
AQDB_VALUE *AQDB_Value_fromDouble(double i);

/**
 * This function returns the value as a double.
 * You should not feed another AQDB_VALUE from this double, because the
 * conversion from an AQDB_VALUE to a double might be lossy!
 */
AQDB_API
double AQDB_Value_GetValueAsDouble(const AQDB_VALUE *v);


/**
 * You should not use a double retrieved via
 * @ref AQDB_Value_GetValueAsDouble as an argument to this function, because
 * the conversion from AQDB_VALUE to double to AQDB_VALUE might change the
 * real value.
 */
AQDB_API
void AQDB_Value_SetValueFromDouble(AQDB_VALUE *v, double i);

AQDB_API
void AQDB_Value_SetZero(AQDB_VALUE *v);

AQDB_API
int AQDB_Value_IsZero(const AQDB_VALUE *v);

AQDB_API
int AQDB_Value_IsNegative(const AQDB_VALUE *v);

AQDB_API
int AQDB_Value_IsPositive(const AQDB_VALUE *v);

AQDB_API
int AQDB_Value_Compare(const AQDB_VALUE *v1, const AQDB_VALUE *v2);

AQDB_API
int AQDB_Value_AddValue(AQDB_VALUE *v1, const AQDB_VALUE *v2);

AQDB_API
int AQDB_Value_SubValue(AQDB_VALUE *v1, const AQDB_VALUE *v2);

AQDB_API
int AQDB_Value_MultValue(AQDB_VALUE *v1, const AQDB_VALUE *v2);

AQDB_API
int AQDB_Value_DivValue(AQDB_VALUE *v1, const AQDB_VALUE *v2);

AQDB_API
int AQDB_Value_Negate(AQDB_VALUE *v);

AQDB_API
void AQDB_Value_Dump(const AQDB_VALUE *v, FILE *f, unsigned int indent);

#ifdef __cplusplus
}
#endif


#endif /* AQDB_VALUE_H */








